import arcpy

inTable = arcpy.GetParameterAsText(0)
addFieldsTable = arcpy.GetParameterAsText(1)

try:
    cur = arcpy.SearchCursor(addFieldsTable)
    #create an empty list to hold lists of field parameters
    addFieldParameterList = []
    arcpy.AddMessage('\n**********************************************')

    #loop through each row and pull the field information from each column.
    for row in cur:
        #in each row append a new item to the addFieldParams list. Each item is actually a
        # list of more items. the end result is a 'master list' containing individual lists
        #of field properties for each field.
        addFieldParameterList.append((row.fieldname, row.fieldtype, row.precision,
                                      row.scale, row.length, row.alias, row.isnullable,
                                      row.domain))
    del cur

    # for each group of parameters we need to do some processing and add a field.
    for parameterGroup in addFieldParameterList:

        #set the parameters for the add field tool from the parameters one group at a time
        fieldname, fieldtype, precision, scale, length, alias, isnullable, domain = parameterGroup

        #tool validation will take care of errors for the required fields.

        #If the fields table is DBF, we want to convert the alias and nullable parameters = 0 to None.
        if '.dbf' in addFieldsTable:
            if alias == 0: alias = None
            if isnullable == 0: isnullable = None
            if domain == 0: domain = None

        #if it's a non numeric field set the precision and scale to None
        if fieldtype.lower() in ('text', 'date', 'blob', 'raster', 'guid'):
            precision = None
            scale = None
        #if it's a short or long field set the scale to None
        elif fieldtype.lower() in ('short', 'long'):
            scale = None
            length = None
        #if it's a float or double field set the length to None
        else:
            length = None

        # Check to see if the field exists, if it does delete it.
        if fieldname.lower() in [field.name.lower() for field in arcpy.ListFields(inTable)]:
            arcpy.DeleteField_management(inTable, fieldname)
            arcpy.AddField_management(inTable, fieldname, fieldtype, precision, scale,
                                      length, alias, isnullable, '', domain)
            arcpy.AddMessage('Successfully added field: %s to %s' %
                             (fieldname, arcpy.Describe(inTable).name))
        else:
            arcpy.AddField_management(inTable, fieldname, fieldtype, precision, scale,
                                      length, alias, isnullable, '', domain)
            arcpy.AddMessage('Successfully added field: %s to %s' %
                             (fieldname, arcpy.Describe(inTable).name))

    arcpy.AddMessage('**********************************************\n')

except:
    arcpy.AddMessage('Table to fields script FAILED \n Error Messages Below \n')
    arcpy.AddMessage(arcpy.GetMessages() + '\n')
    arcpy.AddMessage('End of table to fields error messages \n**********************************************\n')



