#import required modules
import arcpy, os

#Get variables from user input
inWksp = arcpy.GetParameterAsText(0)
outFolder = arcpy.GetParameterAsText(1)
outFormat = arcpy.GetParameterAsText(2)
####
##inWksp = r"C:\_AGC\Demos\4_Documentation\GIS.gdb"
##outFolder = r"C:\_AGC\Demos\4_Documentation\csvfiles"
##outFormat = 'EXCEL'

'''Function used to read csv files and write to xls file
this function requires the xlwt module to be installed
on the client machine.
Flags are set in the code to check for the module.
If the module is not installed the core code will not run'''
def writeExcel(inFolder, outFolder, outName):
  #These modules are only needed when writing xls
  import glob, csv
  wb = xlwt.Workbook()
  #Loop through all of the csv files in the input folder
  for filename in glob.glob(os.path.join(inFolder,"*.csv")):
      #Get the name of the csv file
      (f_path, f_name) = os.path.split(filename)
      (f_short_name, f_extension) = os.path.splitext(f_name)
      #Create a new sheet in the work boon named after the csv
      ws = wb.add_sheet(f_short_name)
      #Read info from csv file.
      csvLines = csv.reader(open(filename, 'rb'))
      for rowx, row in enumerate(csvLines):
          for colx, value in enumerate(row):
              #Convert to int if possible
              try:
                  if int(value):
                      value = int(value)
              except:
                  pass
              #Write the row value.
              ws.write(rowx, colx, value)
  #Save the xls file
  wb.save(os.path.join(inFolder, outName))

'''Function used to write feature class info to a csv file'''
def writeFcInfo(featureClassList, csvFile):
  csvFile.write('Feature Class Info:\n')
  #Iterate through each feature class
  for fc in featureClassList:
    #Describe the fc and write info.
    descFC = arcpy.Describe(fc)
    csvFile.write('Name:,{0}\n'.format(fc))
    csvFile.write('Shape Type:,{0}\n'.format(descFC.shapeType))
    csvFile.write('Feature Type:,{0}\n'.format(descFC.featureType))
    #Get a count of all the records in the feature class.
    recordcount = arcpy.GetCount_management(fc)
    csvFile.write('Record Count:, {0}\n'.format(recordcount))
    #Get a list of fields.
    fcFields = arcpy.ListFields(fc)
    csvFile.write('# of fields:,{0}\n'.format(len(fcFields)))
    #Intialize strings to store names and types
    fNamesString = ''
    fTypesString = ''
    #Generate a comma delimited string with names and types.
    for n in fcFields:
      fNamesString += n.name + ','
    for t in fcFields:
      fTypesString += t.type + ','
    csvFile.write('Field Types:,{0} \n'.format(fTypesString))
    csvFile.write('Field Names:,{0} \r\n'.format(fNamesString))
  return

'''Function used to write table info to a csv file'''
def writeTableInfo(tableList, csvFile):
  csvFile.write('Table Info:\n')
  #Iterate through each table
  for t in tableList:
    #Describe the table and write info.
    descFC = arcpy.Describe(t)
    csvFile.write('Name:,{0}\n'.format(t))
    #Get a count of all the records in the table.
    recordcount = arcpy.GetCount_management(t)
    csvFile.write('Record Count:, {0}\n'.format(recordcount))
    #Get a list of fields.
    tableFields = arcpy.ListFields(t)
    csvFile.write('# of fields:,{0}\n'.format(len(tableFields)))
    #Initialize strings to store names and types.
    tNamesString = ''
    tTypesString = ''
    #Generate a comma delimited string with names and types.
    for n in tableFields:
      tNamesString += n.name + ','
    for t in tableFields:
      tTypesString += t.type + ','
    csvFile.write('Field Types:,{0} \n'.format(tTypesString))
    csvFile.write('Field Names:,{0} \r\n'.format(tNamesString))
  return

'''Function used to write relatinship class info to a csv file'''
def writeRcInfo(relationshipClassList, csvFile):
  #Iterate through each relationship class
  for rc in relationshipClassList:
    #Write the name
    csvFile.write('Dataset Name:,{0}\n'.format(rc.name))
    csvFile.write('Dataset Type:,Relationship Class\n')
    #Intialize strings to store origin/destination info
    destClasses = ''
    originClasses = ''
    #Generate a comma delimited string with origin and destination names.
    for dcName in rc.destinationClassNames:
      destClasses += ',' + dcName
    for ocName in rc.originClassNames:
      originClasses += ',' + ocName
    csvFile.write('Destination Classes:{0}\n'.format(destClasses))
    csvFile.write('Origin Classes:{0}\r\n'.format(originClasses))
  return

'''Function used to write all info about data in a fds to a csv file'''
def createDatasetCSV(inDataset,outLocation, outName=""):
  try:
    #Keep track of the original workspace environment
    origWorkspace = arcpy.env.workspace
    #Describe the input dataset
    desc = arcpy.Describe(inDataset)
    if outName =='':
      outName = desc.Name
    #Create the output csv file
    outCSV = open(outLocation + os.sep + outName +'.csv', 'w')
    #Set the new workspace to the input dataset
    arcpy.env.workspace = inDataset
    #Generate a list of relationship classes, datasets and feature classes
    rcList = [c for c in desc.children if c.datatype == "RelationshipClass"]
    dsList = arcpy.ListDatasets()
    fcList = arcpy.ListFeatureClasses()
    #Write the info to the csv
    outCSV.write('Workspace Type:, Feature Dataset \n Name:, {0}\n# of feature classes:, {1}\n'.format(desc.Name,len(fcList)))
    outCSV.write('# of controller datasets:, {0}\r\n'.format(len(dsList)))
    #Iterate through the relationships and datasets and write info.
    if len(dsList) > 0 or len(rcList) > 0:
      outCSV.write('Dataset Info:\n')
      if len(dsList) > 0:
        for ds in dsList:
          dsDesc = arcpy.Describe(ds)
          if dsDesc.datasetType == 'GeometricNetwork':
            outCSV.write('Dataset Name:, {0}\n'.format(ds))
            outCSV.write('Dataset Type:,Geometric Network\n')
            children = ''
            for c in dsDesc.featureClassNames:
              children += ',' + c
            outCSV.write('Classes in GN:{0}\n'.format(children))
          elif dsDesc.datasetType == 'Topology':
            outCSV.write('Dataset Name:,{0}\n'.format(ds))
            outCSV.write('Dataset Type:,Topology\n')
            outCSV.write('# of classes:,{0}\n'.format(len(dsDesc.featureClassNames)))
            children = ''
            for c in dsDesc.featureClassNames:
              children += ',' + c
            outCSV.write('Classes in Topology: {0}\n'.format(children))
          elif dsDesc.datasetType == 'NetworkDataset':
            outCSV.write('Dataset Name:,{0}\n'.format(ds))
            outCSV.write('Dataset Type:,Network Dataset\n')
            outCSV.write('# of sources:,{0}\n'.format(len(dsDesc.sources)))
            sources = ''
            sourceType = ''
            for s in dsDesc.sources:
              sources += ',' + s.name
              sourceType += ',' + s.sourceType
            outCSV.write('Sources: {0}\n'.format(sources))
            outCSV.write('Source types: {0}\n'.format(sourceType))
          elif dsDesc.datasetType == 'CadastralFabric':
            outCSV.write('Dataset Name:,{0}\n'.format(ds))
            outCSV.write('Dataset Type:,Cadastral Fabric\n')
          elif dsDesc.datasetType == 'Terrain':
            outCSV.write('Dataset Name:,{0}\n'.format(ds))
            outCSV.write('Dataset Type:,Terrain\n')
          else:
            #If the dataset is not one of the above we will just write it's name.
            outCSV.write('Name:,{0}\n'.format(ds))
      #Write the relationship class info to file
      if len(rcList) > 0:
        writeRcInfo(rcList, outCSV)
      outCSV.write('\r\n')
    if len(fcList) > 0:
      writeFcInfo(fcList, outCSV)
    #Close the csv file for writing
    outCSV.close()
    arcpy.env.workspace = origWorkspace

    return outCSV
  except:
    arcpy.AddMessage('fatal error')

try:
  #Initialize the excel writing flag.
  excelFlag = False

  #Test to see if the user wants to write excel
  #if so, import the required module and set the flag.
  if outFormat.lower() in ['excel']:
    try:
      import xlwt
      excelFlag = True
    #if the import fails return a warning message to the user and stop processing.
    except:
      arcpy.AddMessage("\n")
      arcpy.AddMessage('#### WARNING ####')
      arcpy.AddMessage('Necessary modules are not installed')
      arcpy.AddMessage('Please download and install xlwt from the following location.\r\nhttp://pypi.python.org/pypi/xlwt')
      arcpy.AddMessage('Once downloaded, read the README.html for instal instructions.')
      arcpy.AddMessage("\n")

  #If the user does not want excel we can just continue on.
  if excelFlag == True or outFormat.lower() == 'csv':
    #Set the workspace to the input fds or gdb.
    arcpy.env.workspace = inWksp
    #Describe the workspace.
    wkspDesc = arcpy.Describe(inWksp)
    #if the workspace is a feature dataset we just need to call the create datasetcsv function
    if wkspDesc.dataType == 'FeatureDataset':
      createDatasetCSV(inWksp,outFolder)
      #If the user wants an excel file we merge the csv files into an xls
      if excelFlag == True:
        writeExcel(outFolder, outFolder, wkspDesc.name + '.xls')
    #If the user provides a geodatabase we have more work to do.
    elif wkspDesc.dataType == 'Workspace' and wkspDesc.workspaceType != 'FileSystem':
      #Generate lists of data.
      fdsList = arcpy.ListDatasets('', 'Feature')
      fcList = arcpy.ListFeatureClasses()
      tList = arcpy.ListTables()
      rcList = [c for c in wkspDesc.children if c.datatype == "RelationshipClass"]

      #Create the csv file for the root gdb and write info
      gdbCsvName = 'GDB_Root'
      gdbCSV = open(outFolder + os.sep + gdbCsvName +'.csv', 'w')
      gdbCSV.write('Name:,{0}\n'.format(wkspDesc.baseName))
      gdbCSV.write("# of fds's:,{0}\n".format(len(fdsList)))
      gdbCSV.write('# of stand alone fc:,{0}\n'.format(len(fcList)))
      gdbCSV.write('# of stand alone tables:,{0}\n'.format(len(tList)))
      gdbCSV.write('# of stand alone relationships:,{0}\r\n'.format(len(rcList)))
      #Write info for the various datasets.
      if len(rcList) > 0:
        gdbCSV.write('Relationship Class Info:')
        writeRcInfo(rcList, gdbCSV)
      if len(fcList) > 0:
        writeFcInfo(fcList, gdbCSV)
      if len(tList) > 0:
        writeTableInfo(tList,gdbCSV)
      gdbCSV.close()
      if len(fdsList) > 0:
        for fds in fdsList:
          createDatasetCSV(fds, outFolder)

      #If the user wants an excel file we merge the csv files into an xls
      if excelFlag == True:
        writeExcel(outFolder, outFolder, wkspDesc.baseName + '.xls')
    else:
      #If the user somehow provides an invalid datatype we provide an error message.
      arcpy.AddMessage('Invalid input type: Must be Feature Dataset or Geodatabase')

except:
  arcpy.AddMessage(arcpy.GetMessages())
